/*
  Example of PARTIAL class specialization
*/

#include <iostream>
#include <fstream>
#include <cmath>

/*
  Define the class vec parametrized with integer N and type T.
 */
template <typename T, int N>
class vec {
public:
  typedef T valueType ;
private:
  T vals[N] ;
public:
  vec()
  { std::fill( vals, vals + N, 0 ) ; }
  
  vec<T,N> const & operator = ( vec<T,N> const & v ) {
    std::copy( v.vals, v.vals + N, vals ) ;
    return *this ;
  }
  
  // many other operators may be defined

  T const & operator [] ( int i ) const { return vals[i] ; }
  T       & operator [] ( int i )       { return vals[i] ; }
  
} ;

/*
  Define external operator + for the class vec parametrized with integer N and type T.
 */
template <typename T, int N>
vec<T,N>
operator + ( vec<T,N> const & a, vec<T,N> const & b ) {
  std::cout << "using operator +\n" ;
  vec<T,N> res ;
  for ( int i = 0 ; i < N ; ++i )
    res[i] = a[i] + b[i] ;
  return res ;
}

// add a partial specialization of operator + 
// for improving performance
template <typename T>
vec<T,2>
operator + ( vec<T,2> const & a, vec<T,2> const & b ) {
  std::cout << "using operator + specialized for N=2\n" ;
  vec<T,2> res ;
  res[0] = a[0] + b[0] ;
  res[1] = a[1] + b[1] ;
  return res ;
}

// add a partial specialization of operator + 
// for improving performance
template <typename T>
vec<T,3>
operator + ( vec<T,3> const & a, vec<T,3> const & b ) {
  std::cout << "using operator + specialized for N=3\n" ;
  vec<T,3> res ;
  res[0] = a[0] + b[0] ;
  res[1] = a[1] + b[1] ;
  return res ;
}

/*
  Define external function "dot" for the class vec parametrized with integer N and type T.
 */
template <typename T, int N>
vec<T,N>
dot( vec<T,N> const & a, vec<T,N> const & b ) {
  typename vec<T,N>::valueType res ;
  res = 0 ;
  for ( int i = 0 ; i < N ; ++i )
    res += a[i] + b[i] ;
  return res ;
}

/*
  Define external operator << for the class vec parametrized with integer N and type T.
 */
template <typename T, int N>
std::ostream &
operator << ( std::ostream & s, vec<T,N> const & a ) {
  s << "[ " ;
  for ( int i = 0 ; i < N ; ++i )
    s << a[i] << " " ;
  s << "]" ;
  return s ;
}

int
main() {
  vec<double,2> a2, b2 ;
  vec<double,3> a3, b3 ;
  vec<double,4> a4, b4 ;

  a2 = a2 + b2 ; 
  a3 = a3 + b3 ;
  a4 = a4 + b4 ;

  return 0 ;
}
