/*
  Example of class specialization
*/

#include <iostream>
#include <fstream>
#include <cmath>

/*
  Define the class vec parametrized with integer N.
 */
template <int N>
class vec {
  double vals[N] ;
public:
  vec()
  { std::fill( vals, vals + N, 0 ) ; }
  
  vec<N> const & operator = ( vec<N> const & v ) {
    std::copy( v.vals, v.vals + N, vals ) ;
  }
  
  // many other operators may be defined

  double const & operator [] ( int i ) const { return vals[i] ; }
  double       & operator [] ( int i )       { return vals[i] ; }
  
} ;

/*
  Define external operator + for the class vec parametrized with integer N.
 */
template <int N>
vec<N>
operator + ( vec<N> const & a, vec<N> const & b ) {
  vec<N> res ;
  for ( int i = 0 ; i < N ; ++i )
    res[i] = a[i] + b[i] ;
  return res ;
}

/*
  Define external operator << for the class vec parametrized with integer N.
 */
template <int N>
std::ostream &
operator << ( std::ostream & s, vec<N> const & a ) {
  s << "[ " ;
  for ( int i = 0 ; i < N ; ++i )
    s << a[i] << " " ;
  s << "]" ;
  return s ;
}

/*
  driver program to test specialization
 */
int
main() {
  vec<5>  a5 ;
  vec<10> a10 ;

  a5[3] = 1 ;
  a5[2] = 12 ;
  a5[1] = 3 ;
  std::cout << a5 << "\n" ;

  a10[3] = 1 ;
  a10[2] = 12 ;
  a10[1] = 3 ;
  std::cout << a10 << "\n" ;

  return 0 ;
}
