/*
  Example of class specialization
*/

#include <iostream>
#include <fstream>
#include <cmath>

/*
  Define the class vec parametrized with integer N and type T.
 */
template <typename T, int N>
class vec {
public:
  // define the type valueType as an alias of type T.
  // this will be accessible in this way
  // VEC::valueType
  // if VEC is an alias of vec<T,N>
  typedef T valueType ;
private:
  T vals[N] ;
public:
  vec()
  { std::fill( vals, vals + N, 0 ) ; }
  
  vec<T,N> const & operator = ( vec<T,N> const & v ) {
    std::copy( v.vals, v.vals + N, vals ) ;
  }
  
  // many other operators may be defined

  T const & operator [] ( int i ) const { return vals[i] ; }
  T       & operator [] ( int i )       { return vals[i] ; }
  
} ;

/*
  Define external operator + for the class vec parametrized with integer N and type T.
 */
template <typename T, int N>
vec<T,N>
operator + ( vec<T,N> const & a, vec<T,N> const & b ) {
  vec<T,N> res ;
  for ( int i = 0 ; i < N ; ++i )
    res[i] = a[i] + b[i] ;
  return res ;
}

/*
  Define external function "dot" for the class vec parametrized with integer N and type T.
 */
template <typename T, int N>
vec<T,N>
dot( vec<T,N> const & a, vec<T,N> const & b ) {
  typename vec<T,N>::valueType res ;
  res = 0 ;
  for ( int i = 0 ; i < N ; ++i )
    res += a[i] + b[i] ;
  return res ;
}

/*
  Define external operator << for the class vec parametrized with integer N and type T.
 */
template <typename T, int N>
std::ostream &
operator << ( std::ostream & s, vec<T,N> const & a ) {
  s << "[ " ;
  for ( int i = 0 ; i < N ; ++i )
    s << a[i] << " " ;
  s << "]" ;
  return s ;
}

int
main() {
  vec<double,5>  a5 ;
  vec<float,10> a10 ;
  
  vec<double,5>::valueType scalar ; // scalar is of type double!.

  a5[3] = 1.2 ;
  a5[2] = 12 ;
  a5[1] = 3 ;
  std::cout << a5 << "\n" ;

  a10[3] = 1.2 ;
  a10[2] = 12 ;
  a10[1] = 3 ;
  std::cout << a10 << "\n" ;

  return 0 ;
}
